@@ -33,7 +33,7 @@ use crate::{
 };
 use std::collections::{HashMap, HashSet};
 
-use autocxx_parser::{IncludeCppConfig, UnsafePolicy};
+use autocxx_parser::{ExternCppType, IncludeCppConfig, UnsafePolicy};
 use function_wrapper::{CppFunction, CppFunctionBody, TypeConversionPolicy};
 use itertools::Itertools;
 use proc_macro2::Span;
@@ -66,7 +66,7 @@ use super::{
     doc_label::make_doc_attrs,
     pod::{PodAnalysis, PodPhase},
     tdef::TypedefAnalysis,
-    type_converter::Annotated,
+    type_converter::{Annotated, PointerTreatment},
 };
 
 #[derive(Clone, Debug)]
@@ -177,7 +177,7 @@ pub(crate) struct ArgumentAnalysis {
     pub(crate) conversion: TypeConversionPolicy,
     pub(crate) name: Pat,
     pub(crate) self_type: Option<(QualifiedName, ReceiverMutability)>,
-    pub(crate) was_reference: bool,
+    pub(crate) has_lifetime: bool,
     pub(crate) deps: HashSet<QualifiedName>,
     pub(crate) requires_unsafe: UnsafetyNeeded,
     pub(crate) is_placement_return_destination: bool,
@@ -188,15 +188,17 @@ struct ReturnTypeAnalysis {
     conversion: Option<TypeConversionPolicy>,
     was_reference: bool,
     deps: HashSet<QualifiedName>,
+    placement_param_needed: Option<(FnArg, ArgumentAnalysis)>,
 }
 
 impl Default for ReturnTypeAnalysis {
     fn default() -> Self {
         Self {
             rt: parse_quote! {},
-            conversion: Default::default(),
-            was_reference: Default::default(),
+            conversion: None,
+            was_reference: false,
             deps: Default::default(),
+            placement_param_needed: None,
         }
     }
 }
@@ -273,6 +275,7 @@ pub(crate) struct FnAnalyzer<'a> {
     type_converter: TypeConverter<'a>,
     bridge_name_tracker: BridgeNameTracker,
     pod_safe_types: HashSet<QualifiedName>,
+    moveit_safe_types: HashSet<QualifiedName>,
     config: &'a IncludeCppConfig,
     overload_trackers_by_mod: HashMap<Namespace, OverloadTracker>,
     subclasses_by_superclass: HashMap<QualifiedName, Vec<SubclassName>>,
@@ -295,6 +298,7 @@ impl<'a> FnAnalyzer<'a> {
             config,
             overload_trackers_by_mod: HashMap::new(),
             pod_safe_types: Self::build_pod_safe_type_set(&apis),
+            moveit_safe_types: Self::build_correctly_sized_type_set(&apis),
             subclasses_by_superclass: subclass::subclasses_by_superclass(&apis),
             nested_type_name_map: Self::build_nested_type_map(&apis),
             generic_types: Self::build_generic_type_set(&apis),
@@ -346,6 +350,26 @@ impl<'a> FnAnalyzer<'a> {
             .collect()
     }
 
+    /// Return the set of 'moveit safe' types. That must include only types where
+    /// the size is known to be correct.
+    fn build_correctly_sized_type_set(apis: &ApiVec<PodPhase>) -> HashSet<QualifiedName> {
+        apis.iter()
+            .filter(|api| {
+                matches!(
+                    api,
+                    Api::Struct { .. }
+                        | Api::Enum { .. }
+                        | Api::ExternCppType {
+                            details: ExternCppType { opaque: false, .. },
+                            ..
+                        }
+                )
+            })
+            .map(|api| api.name().clone())
+            .chain(known_types().get_moveit_safe_types())
+            .collect()
+    }
+
     fn build_generic_type_set(apis: &ApiVec<PodPhase>) -> HashSet<QualifiedName> {
         apis.iter()
             .filter_map(|api| match api {
@@ -384,11 +408,9 @@ impl<'a> FnAnalyzer<'a> {
         &mut self,
         ty: Box<Type>,
         ns: &Namespace,
-        convert_ptrs_to_references: bool,
+        pointer_treatment: PointerTreatment,
     ) -> Result<Annotated<Box<Type>>, ConvertError> {
-        let ctx = TypeConversionContext::CxxOuterType {
-            convert_ptrs_to_references,
-        };
+        let ctx = TypeConversionContext::CxxOuterType { pointer_treatment };
         let mut annotated = self.type_converter.convert_boxed_type(ty, ns, &ctx)?;
         self.extra_apis.append(&mut annotated.extra_apis);
         Ok(annotated)
@@ -419,7 +441,7 @@ impl<'a> FnAnalyzer<'a> {
         param_details: &[ArgumentAnalysis],
         kind: &FnKind,
     ) -> UnsafetyNeeded {
-        let unsafest_non_self_param = UnsafetyNeeded::from_param_details(param_details, true);
+        let unsafest_non_placement_param = UnsafetyNeeded::from_param_details(param_details, true);
         let unsafest_param = UnsafetyNeeded::from_param_details(param_details, false);
         match kind {
             // Trait unsafety must always correspond to the norms for the
@@ -437,11 +459,11 @@ impl<'a> FnAnalyzer<'a> {
                 _ => unsafest_param,
             },
             _ if self.unsafe_policy == UnsafePolicy::AllFunctionsUnsafe => UnsafetyNeeded::Always,
-            _ => match unsafest_non_self_param {
+            _ => match unsafest_non_placement_param {
                 UnsafetyNeeded::Always => UnsafetyNeeded::Always,
                 UnsafetyNeeded::JustBridge => match unsafest_param {
                     UnsafetyNeeded::Always => UnsafetyNeeded::JustBridge,
-                    _ => unsafest_non_self_param,
+                    _ => unsafest_non_placement_param,
                 },
                 UnsafetyNeeded::None => match unsafest_param {
                     UnsafetyNeeded::Always => UnsafetyNeeded::JustBridge,
@@ -722,6 +744,7 @@ impl<'a> FnAnalyzer<'a> {
                     &fun.synthesized_this_type,
                     &fun.references,
                     true,
+                    false,
                     None,
                     sophistication,
                     false,
@@ -1003,6 +1026,7 @@ impl<'a> FnAnalyzer<'a> {
                     None,
                     sophistication,
                     true,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1021,6 +1045,7 @@ impl<'a> FnAnalyzer<'a> {
                     Some(RustConversionType::FromTypeToPtr),
                     sophistication,
                     false,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1044,6 +1069,7 @@ impl<'a> FnAnalyzer<'a> {
                     Some(RustConversionType::FromPinMaybeUninitToPtr),
                     sophistication,
                     false,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1068,6 +1094,7 @@ impl<'a> FnAnalyzer<'a> {
                     Some(RustConversionType::FromPinMaybeUninitToPtr),
                     sophistication,
                     false,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
                 self.reanalyze_parameter(
@@ -1080,14 +1107,13 @@ impl<'a> FnAnalyzer<'a> {
                     Some(RustConversionType::FromPinMoveRefToPtr),
                     sophistication,
                     false,
+                    true,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
             _ => {}
         }
 
-        let requires_unsafe = self.should_be_unsafe(&param_details, &kind);
-
         // Now we can add context to the error, check for a variety of error
         // cases. In each case, we continue to record the API, because it might
         // influence our later decisions to generate synthetic constructors
@@ -1121,16 +1147,6 @@ impl<'a> FnAnalyzer<'a> {
             set_ignore_reason(ConvertError::RValueReturn)
         } else if fun.is_deleted {
             set_ignore_reason(ConvertError::Deleted)
-        } else if !fun.references.rvalue_ref_params.is_empty()
-            && !matches!(
-                kind,
-                FnKind::TraitMethod {
-                    kind: TraitMethodKind::MoveConstructor,
-                    ..
-                }
-            )
-        {
-            set_ignore_reason(ConvertError::RValueParam)
         } else {
             match kind {
                 FnKind::Method {
@@ -1196,9 +1212,10 @@ impl<'a> FnAnalyzer<'a> {
                 })),
                 was_reference: false,
                 deps: std::iter::once(impl_for).cloned().collect(),
+                placement_param_needed: None,
             }
         } else {
-            self.convert_return_type(&fun.output, ns, &fun.references)
+            self.convert_return_type(&fun.output, ns, &fun.references, sophistication)
                 .unwrap_or_else(|err| {
                     set_ignore_reason(err);
                     ReturnTypeAnalysis::default()
@@ -1207,7 +1224,17 @@ impl<'a> FnAnalyzer<'a> {
         let mut deps = params_deps;
         deps.extend(return_analysis.deps.drain());
 
-        let num_input_references = param_details.iter().filter(|pd| pd.was_reference).count();
+        // Sometimes, the return type will actually be a value type
+        // for which we instead want to _pass_ a pointer into which the value
+        // can be constructed. Handle that case here.
+        if let Some((extra_param, extra_param_details)) = return_analysis.placement_param_needed {
+            param_details.push(extra_param_details);
+            params.push(extra_param);
+        }
+
+        let requires_unsafe = self.should_be_unsafe(&param_details, &kind);
+
+        let num_input_references = param_details.iter().filter(|pd| pd.has_lifetime).count();
         if num_input_references != 1 && return_analysis.was_reference {
             // cxx only allows functions to return a reference if they take exactly
             // one reference as a parameter. Let's see...
@@ -1316,10 +1343,12 @@ impl<'a> FnAnalyzer<'a> {
             };
             // Now modify the cxx::bridge entry we're going to make.
             if let Some(ref conversion) = ret_type_conversion {
-                let new_ret_type = conversion.unconverted_rust_type();
-                ret_type = parse_quote!(
-                    -> #new_ret_type
-                );
+                if conversion.populate_return_value() {
+                    let new_ret_type = conversion.unconverted_rust_type();
+                    ret_type = parse_quote!(
+                        -> #new_ret_type
+                    );
+                }
             }
 
             // Amend parameters for the function which we're asking cxx to generate.
@@ -1419,6 +1448,7 @@ impl<'a> FnAnalyzer<'a> {
         force_rust_conversion: Option<RustConversionType>,
         sophistication: TypeConversionSophistication,
         construct_into_self: bool,
+        is_move_constructor: bool,
     ) -> Result<(), ConvertError> {
         self.convert_fn_arg(
             fun.inputs.iter().nth(param_idx).unwrap(),
@@ -1427,6 +1457,7 @@ impl<'a> FnAnalyzer<'a> {
             &fun.synthesized_this_type,
             &fun.references,
             false,
+            is_move_constructor,
             force_rust_conversion,
             sophistication,
             construct_into_self,
@@ -1582,6 +1613,7 @@ impl<'a> FnAnalyzer<'a> {
         virtual_this: &Option<QualifiedName>,
         references: &References,
         treat_this_as_reference: bool,
+        is_move_constructor: bool,
         force_rust_conversion: Option<RustConversionType>,
         sophistication: TypeConversionSophistication,
         construct_into_self: bool,
@@ -1591,8 +1623,7 @@ impl<'a> FnAnalyzer<'a> {
                 let mut pt = pt.clone();
                 let mut self_type = None;
                 let old_pat = *pt.pat;
-                let mut treat_as_reference = false;
-                let mut treat_as_rvalue_reference = false;
+                let mut pointer_treatment = PointerTreatment::Pointer;
                 let mut is_placement_return_destination = false;
                 let new_pat = match old_pat {
                     syn::Pat::Ident(mut pp) if pp.ident == "this" => {
@@ -1637,38 +1668,38 @@ impl<'a> FnAnalyzer<'a> {
                         is_placement_return_destination = construct_into_self;
                         if treat_this_as_reference {
                             pp.ident = Ident::new("self", pp.ident.span());
-                            treat_as_reference = true;
+                            pointer_treatment = PointerTreatment::Reference;
                         }
                         syn::Pat::Ident(pp)
                     }
                     syn::Pat::Ident(pp) => {
                         validate_ident_ok_for_cxx(&pp.ident.to_string())?;
-                        treat_as_reference = references.ref_params.contains(&pp.ident);
-                        treat_as_rvalue_reference =
-                            references.rvalue_ref_params.contains(&pp.ident);
+                        pointer_treatment = references.param_treatment(&pp.ident);
                         syn::Pat::Ident(pp)
                     }
                     _ => old_pat,
                 };
-                let annotated_type = self.convert_boxed_type(pt.ty, ns, treat_as_reference)?;
-                let new_ty = annotated_type.ty;
-                let subclass_holder = match &annotated_type.kind {
-                    type_converter::TypeKind::SubclassHolder(holder) => Some(holder),
-                    _ => None,
-                };
+                let is_placement_return_destination = is_placement_return_destination
+                    || matches!(
+                        force_rust_conversion,
+                        Some(RustConversionType::FromPlacementParamToNewReturn)
+                    );
+                let annotated_type = self.convert_boxed_type(pt.ty, ns, pointer_treatment)?;
                 let conversion = self.argument_conversion_details(
-                    &new_ty,
-                    &subclass_holder.cloned(),
-                    treat_as_rvalue_reference,
+                    &annotated_type,
+                    is_move_constructor,
                     force_rust_conversion,
                     sophistication,
                 );
+                let new_ty = annotated_type.ty;
                 pt.pat = Box::new(new_pat.clone());
                 pt.ty = new_ty;
                 let requires_unsafe =
-                    if matches!(annotated_type.kind, type_converter::TypeKind::Pointer) {
+                    if matches!(annotated_type.kind, type_converter::TypeKind::Pointer)
+                        && !is_placement_return_destination
+                    {
                         UnsafetyNeeded::Always
-                    } else if conversion.bridge_unsafe_needed() {
+                    } else if conversion.bridge_unsafe_needed() || is_placement_return_destination {
                         UnsafetyNeeded::JustBridge
                     } else {
                         UnsafetyNeeded::None
@@ -1679,7 +1710,7 @@ impl<'a> FnAnalyzer<'a> {
                         self_type,
                         name: new_pat,
                         conversion,
-                        was_reference: matches!(
+                        has_lifetime: matches!(
                             annotated_type.kind,
                             type_converter::TypeKind::Reference
                                 | type_converter::TypeKind::MutableReference
@@ -1696,12 +1727,20 @@ impl<'a> FnAnalyzer<'a> {
 
     fn argument_conversion_details(
         &self,
-        ty: &Type,
-        is_subclass_holder: &Option<Ident>,
-        is_rvalue_ref: bool,
+        annotated_type: &Annotated<Box<Type>>,
+        is_move_constructor: bool,
         force_rust_conversion: Option<RustConversionType>,
         sophistication: TypeConversionSophistication,
     ) -> TypeConversionPolicy {
+        let is_subclass_holder = match &annotated_type.kind {
+            type_converter::TypeKind::SubclassHolder(holder) => Some(holder),
+            _ => None,
+        };
+        let is_rvalue_ref = matches!(
+            annotated_type.kind,
+            type_converter::TypeKind::RValueReference
+        );
+        let ty = &*annotated_type.ty;
         if let Some(holder_id) = is_subclass_holder {
             let subclass = SubclassName::from_holder_name(holder_id);
             return {
@@ -1714,6 +1753,16 @@ impl<'a> FnAnalyzer<'a> {
                     rust_conversion: RustConversionType::ToBoxedUpHolder(subclass),
                 }
             };
+        } else if matches!(
+            force_rust_conversion,
+            Some(RustConversionType::FromPlacementParamToNewReturn)
+        ) && matches!(sophistication, TypeConversionSophistication::Regular)
+        {
+            return TypeConversionPolicy {
+                unwrapped_type: ty.clone(),
+                cpp_conversion: CppConversionType::IgnoredPlacementPtrParameter,
+                rust_conversion: RustConversionType::FromPlacementParamToNewReturn,
+            };
         }
         match ty {
             Type::Path(p) => {
@@ -1754,16 +1803,33 @@ impl<'a> FnAnalyzer<'a> {
                     }
                 }
             }
-            _ => {
-                let cpp_conversion = if is_rvalue_ref {
-                    CppConversionType::FromPtrToMove
+            Type::Ptr(tp) => {
+                let rust_conversion = force_rust_conversion.unwrap_or(RustConversionType::None);
+                if is_move_constructor {
+                    TypeConversionPolicy {
+                        unwrapped_type: ty.clone(),
+                        cpp_conversion: CppConversionType::FromPtrToMove,
+                        rust_conversion,
+                    }
+                } else if is_rvalue_ref {
+                    TypeConversionPolicy {
+                        unwrapped_type: *tp.elem.clone(),
+                        cpp_conversion: CppConversionType::FromPtrToValue,
+                        rust_conversion: RustConversionType::FromRValueParamToPtr,
+                    }
                 } else {
-                    CppConversionType::None
-                };
+                    TypeConversionPolicy {
+                        unwrapped_type: ty.clone(),
+                        cpp_conversion: CppConversionType::None,
+                        rust_conversion,
+                    }
+                }
+            }
+            _ => {
                 let rust_conversion = force_rust_conversion.unwrap_or(RustConversionType::None);
                 TypeConversionPolicy {
                     unwrapped_type: ty.clone(),
-                    cpp_conversion,
+                    cpp_conversion: CppConversionType::None,
                     rust_conversion,
                 }
             }
@@ -1775,41 +1841,85 @@ impl<'a> FnAnalyzer<'a> {
         rt: &ReturnType,
         ns: &Namespace,
         references: &References,
+        sophistication: TypeConversionSophistication,
     ) -> Result<ReturnTypeAnalysis, ConvertError> {
-        let result = match rt {
-            ReturnType::Default => ReturnTypeAnalysis {
-                rt: ReturnType::Default,
-                was_reference: false,
-                conversion: None,
-                deps: HashSet::new(),
-            },
+        Ok(match rt {
+            ReturnType::Default => ReturnTypeAnalysis::default(),
             ReturnType::Type(rarrow, boxed_type) => {
-                // TODO remove the below clone
                 let annotated_type =
-                    self.convert_boxed_type(boxed_type.clone(), ns, references.ref_return)?;
+                    self.convert_boxed_type(boxed_type.clone(), ns, references.return_treatment())?;
                 let boxed_type = annotated_type.ty;
-                let was_reference = matches!(boxed_type.as_ref(), Type::Reference(_));
                 let ty: &Type = boxed_type.as_ref();
-                let conversion = match ty {
-                    Type::Path(p) => {
+                match ty {
+                    Type::Path(p)
+                        if !self
+                            .pod_safe_types
+                            .contains(&QualifiedName::from_type_path(p)) =>
+                    {
                         let tn = QualifiedName::from_type_path(p);
-                        if self.pod_safe_types.contains(&tn) {
-                            TypeConversionPolicy::new_unconverted(ty.clone())
+                        if self.moveit_safe_types.contains(&tn)
+                            && matches!(sophistication, TypeConversionSophistication::Regular)
+                        {
+                            // This is a non-POD type we want to return to Rust as an `impl New` so that callers
+                            // can decide whether to store this on the stack or heap.
+                            // That means, we do not literally _return_ it from C++ to Rust. Instead, our call
+                            // from Rust to C++ will include an extra placement parameter into which the object
+                            // is constructed.
+                            let fnarg = parse_quote! {
+                                placement_return_type: *mut #ty
+                            };
+                            let (fnarg, analysis) = self.convert_fn_arg(
+                                &fnarg,
+                                ns,
+                                "",
+                                &None,
+                                &References::default(),
+                                false,
+                                false,
+                                Some(RustConversionType::FromPlacementParamToNewReturn),
+                                TypeConversionSophistication::Regular,
+                                false,
+                            )?;
+                            ReturnTypeAnalysis {
+                                rt: ReturnType::Default,
+                                conversion: Some(TypeConversionPolicy::new_for_placement_return(
+                                    ty.clone(),
+                                )),
+                                was_reference: false,
+                                deps: annotated_type.types_encountered,
+                                placement_param_needed: Some((fnarg, analysis)),
+                            }
                         } else {
-                            TypeConversionPolicy::new_to_unique_ptr(ty.clone())
+                            // There are some types which we can't currently represent within a moveit::new::New.
+                            // That's either because we are obliged to stick to existing protocols for compatibility
+                            // (CxxString) or because they're a concrete type where we haven't attempted to do
+                            // the analysis to work out the type's size. For these, we always return a plain old
+                            // UniquePtr<T>. These restrictions may be fixed in future.
+                            let conversion =
+                                Some(TypeConversionPolicy::new_to_unique_ptr(ty.clone()));
+                            ReturnTypeAnalysis {
+                                rt: ReturnType::Type(*rarrow, boxed_type),
+                                conversion,
+                                was_reference: false,
+                                deps: annotated_type.types_encountered,
+                                placement_param_needed: None,
+                            }
+                        }
+                    }
+                    _ => {
+                        let was_reference = matches!(boxed_type.as_ref(), Type::Reference(_));
+                        let conversion = Some(TypeConversionPolicy::new_unconverted(ty.clone()));
+                        ReturnTypeAnalysis {
+                            rt: ReturnType::Type(*rarrow, boxed_type),
+                            conversion,
+                            was_reference,
+                            deps: annotated_type.types_encountered,
+                            placement_param_needed: None,
                         }
                     }
-                    _ => TypeConversionPolicy::new_unconverted(ty.clone()),
-                };
-                ReturnTypeAnalysis {
-                    rt: ReturnType::Type(*rarrow, boxed_type),
-                    conversion: Some(conversion),
-                    was_reference,
-                    deps: annotated_type.types_encountered,
                 }
             }
-        };
-        Ok(result)
+        })
     }
 
     /// If a type has explicit constructors, bindgen will generate corresponding
